@@ -48,7 +48,7 @@ def __get_abs_minmax(data, axis, slices, margin=5):
     for a, bba in bb.items():
         res[a] = []
         for each in bba:
-            i = [max(int(min_xs - margin), 0) , int(max_xs + margin)],\
+            i = [max(int(min_xs - margin), 0), int(max_xs + margin)],\
                 [max(int(min_ys - margin), 0), int(max_ys + margin)]
             res[a].append(i)
     return res
@@ -63,9 +63,13 @@ def __maxsize__(data):
     for slice_index in range(0, data.shape[2]):
         test = np.flip(np.swapaxes(np.abs(lambdas['x'](int(slice_index))),
                                    0, 1), 0)
-        black_pixels_mask = np.all(test == 0, axis=-1)
+        if len(data.shape) == 4:
+            black_pixels_mask = np.all(test == [0, 0, 0], axis=-1)
+        else:
+            black_pixels_mask = np.all(test == 0, axis=-1)
 
         size = len(test) - len(black_pixels_mask[black_pixels_mask])
+
         maxsize = max(size, maxsize)
         d.append((slice_index, size))
     return maxsize
