@@ -1,9 +1,8 @@
-import tempfile
-import os
-import os.path as op
+import os as __os__
 
-
-freesurfer_reg_to_native = os.environ.get('FREESURFER_REG_TO_NATIVE', 0) == '1'
+__ev__ = 'FREESURFER_REG_TO_NATIVE'
+__freesurfer_reg_to_native__ = __os__.environ.get(__ev__, None) is not None
+__all__ = []
 
 
 def __is_valid_scan__(xnat_instance, scan):
@@ -63,12 +62,15 @@ def __get_T2__(x, experiment_id, sequence='T2_ALFA1'):
     return t2_t1space
 
 
-def __download_freesurfer6__(x, experiment_id, destination,
-                             resource_name='FREESURFER6',
-                             raw=True, cache=False):
+def __download_freesurfer__(x, experiment_id, destination,
+                            resource_name='FREESURFER7',
+                            raw=True, cache=False, fn='aparc+aseg.mgz'):
+    import os.path as op
     filepaths = []
     e = x.select.experiment(experiment_id)
     r = e.resource(resource_name)
+    if not r.exists():
+        raise Exception('Experiment %s has no resource %s.' % (experiment_id, resource_name))
 
     if raw:
         fp1 = op.join(destination, '%s_T1.nii.gz' % experiment_id)
@@ -79,14 +81,29 @@ def __download_freesurfer6__(x, experiment_id, destination,
     else:
         filepaths.append(None)
 
-    files = ['rawavg.mgz', 'aparc+aseg.mgz'] if freesurfer_reg_to_native\
-        else ['nu.mgz', 'aparc+aseg.mgz']
+    files = ['rawavg.mgz', fn] if __freesurfer_reg_to_native__\
+        else ['nu.mgz', fn]
+    for each in files:
+        c = list(r.files('*%s' % each))
+        if each in ['nu.mgz', 'rawavg.mgz']:  # should come from FREESURFER7
+            r1 = e.resource(resource_name.split('_EXTRAS')[0])
+            c = list(r1.files('*%s' % each))
 
-    for each in files:  # ['orig.mgz', 'aparc+aseg.mgz']:
-        c = list(r.files('*%s' % each))[0]
         fp = op.join(destination, '%s_%s' % (experiment_id, each))
         if not cache:
-            c.get(fp)
+            c[0].get(fp)
+        filepaths.append(fp)
+
+    if fn == 'hypothalamic_subunits_seg.v1.mgz':
+        # Register hypothalamus
+        import nibabel as nib
+        from nilearn.image import resample_img
+        target_affine = nib.load(filepaths[1]).affine  # thalamus
+        img = nib.load(filepaths[2])  # hypothalamus
+        resampled_nii = resample_img(img, target_affine, interpolation='nearest')
+        fp = filepaths[2].replace('.mgz', 'FSvoxelSpace.mgz')
+        resampled_nii.to_filename(fp)
+        filepaths.pop(-1)
         filepaths.append(fp)
 
     from nisnap.utils import aseg
@@ -94,14 +111,14 @@ def __download_freesurfer6__(x, experiment_id, destination,
     bg = filepaths[1]
 
     # Note that the filepaths for these new steps are not stored/returned
-    if freesurfer_reg_to_native:
+    if __freesurfer_reg_to_native__:
         aseg.__preproc_aseg__(aseg_fp, bg, cache=cache)
     aseg.__swap_fs__(aseg_fp, cache=cache)
     aseg.__swap_fs__(bg, cache=cache)
 
     import logging as log
     log.basicConfig(level=log.INFO)
-    log.info('freesurfer_reg_to_native: %s' % freesurfer_reg_to_native)
+    log.info('__freesurfer_reg_to_native__: %s' % __freesurfer_reg_to_native__)
 
     bg = filepaths[0]
 
@@ -109,18 +126,24 @@ def __download_freesurfer6__(x, experiment_id, destination,
 
     # Files were created just before
     # So we only need to fetch their names (with cache=True)
-    if freesurfer_reg_to_native:
+    if __freesurfer_reg_to_native__:
         aseg_fp = aseg.__preproc_aseg__(aseg_fp, bg, cache=True)
     else:
         aseg_fp = aseg.__swap_fs__(aseg_fp, cache=True)
         bg = aseg.__swap_fs__(filepaths[1], cache=True)
 
-    return [bg, aseg_fp]
+    if raw:
+        res = [bg, aseg_fp]
+    else:
+        res = [None, aseg_fp]
+    return res
 
 
 def __download_spm12__(x, experiment_id, destination,
                        resource_name='SPM12_SEGMENT',
                        raw=True, cache=False):
+    import os.path as op
+
     filepaths = []
     e = x.select.experiment(experiment_id)
     r = e.resource(resource_name)
@@ -137,8 +160,8 @@ def __download_spm12__(x, experiment_id, destination,
 
     r = e.resource(resource_name)
     files = ['c1', 'c2', 'c3']
-    if resource_name == 'SPM12_SEGMENT_T2T1':
-        files = ['c1', 'filled_c2', 'c3']
+    if resource_name == 'SPM12_SEGMENT_T1T2':
+        files = ['fixed_c1', 'fixed_c2', 'fixed_c3']
     for each in files:
         c = list(r.files('%s*.nii.gz' % each))[0]
         fp = op.join(destination, '%s_%s_%s.nii.gz'
@@ -152,6 +175,8 @@ def __download_spm12__(x, experiment_id, destination,
 
 def __download_ashs__(x, experiment_id, destination, resource_name='ASHS',
                       raw=True, cache=False):
+    import os.path as op
+
     filepaths = []
     e = x.select.experiment(experiment_id)
     r = e.resource(resource_name)
@@ -214,6 +239,8 @@ def download_resources(config, experiment_id, resource_name,
 
     """
     import pyxnat
+    import os.path as op
+
     x = pyxnat.Interface(config=config)
     filepaths = []
     e = x.select.experiment(experiment_id)
@@ -226,9 +253,30 @@ def download_resources(config, experiment_id, resource_name,
         filepaths = __download_ashs__(x, experiment_id, destination,
                                       resource_name, raw, cache)
 
-    elif 'FREESURFER6' in resource_name:
-        filepaths = __download_freesurfer6__(x, experiment_id, destination,
-                                             resource_name, raw, cache)
+    elif 'FREESURFER' in resource_name and resource_name.endswith('_EXTRAS'):
+        files = ['hypothalamic_subunits_seg.v1.mgz',
+                 'ThalamicNuclei.v12.T1.FSvoxelSpace.mgz',
+                 'brainstemSsLabels.v12.FSvoxelSpace.mgz']
+        filepaths = []
+        for fn in files:
+            fp = __download_freesurfer__(x, experiment_id,
+                                         destination, resource_name,
+                                         raw, cache, fn=fn)
+            if len(filepaths) != 0:
+                fp = [fp[-1]]
+            filepaths.extend(fp)
+
+    elif 'FREESURFER' in resource_name:
+        filepaths = __download_freesurfer__(x, experiment_id, destination,
+                                            resource_name, raw, cache)
+        if resource_name == 'FREESURFER7':
+            fn = '%s.hippoAmygLabels-T1.v21.%s.FSvoxelSpace.mgz'
+            for side in ['lh', 'rh']:
+                for each in ['CA', 'HBT', 'FS60']:
+                    fp = __download_freesurfer__(x, experiment_id, destination,
+                                                 resource_name, raw, cache,
+                                                 fn=fn % (side, each))
+                    filepaths.append(fp[-1])
 
     elif 'CAT12' in resource_name:
         if raw:
@@ -262,8 +310,8 @@ def download_resources(config, experiment_id, resource_name,
 
 
 def plot_segment(config, experiment_id, savefig=None, slices=None,
-                 resource_name='SPM12_SEGMENT',
-                 axes='xyz', raw=True, opacity=10, animated=False,
+                 resource_name='SPM12_SEGMENT', fn='aparc+aseg.mgz',
+                 axes='xyz', raw=True, opacity=70, animated=False,
                  rowsize=None, figsize=None, contours=False, cache=False,
                  samebox=False):
     """Download a given experiment/resource from an XNAT instance and create
@@ -291,6 +339,10 @@ def plot_segment(config, experiment_id, savefig=None, slices=None,
         Name of the resource where the segmentation maps are stored in the XNAT
         instance. Default: SPM12_SEGMENT
 
+    fn: string, optional
+        Only for FREESURFER7_EXTRAS. Defines which segmentation should be
+        rendered (among 'brainstem', 'thalamus', 'hypothalamus') Default: None
+
     axes: string, or a tuple of strings
         Choose the direction of the cuts (among 'x', 'y', or 'z')
 
@@ -342,6 +394,8 @@ def plot_segment(config, experiment_id, savefig=None, slices=None,
         filepaths
 
     """
+    import tempfile
+
     if animated and not raw:
         msg = 'animated cannot be True with raw set to False. ' \
               'Switching raw to True.'
@@ -349,14 +403,29 @@ def plot_segment(config, experiment_id, savefig=None, slices=None,
         log.warning(msg)
         raw = True
 
+    options = ['aparc+aseg.mgz']
+    fn0 = '%s.hippoAmygLabels-T1.v21.%s.FSvoxelSpace.mgz'
+    for side in ['lh', 'rh']:
+        for each in ['CA', 'HBT', 'FS60']:
+            options.append(fn0 % (side, each))
+    if resource_name == 'FREESURFER7_EXTRAS':
+        options = ['hypothalamic_subunits_seg.v1.mgz',
+                   'ThalamicNuclei.v12.T1.FSvoxelSpace.mgz',
+                   'brainstemSsLabels.v12.FSvoxelSpace.mgz']
+        if fn not in options:
+            raise Exception('`fn` should be among %s' % options)
+    elif resource_name.startswith('FREESURFER'):
+        if fn not in options:
+            raise Exception('`fn` should be among %s' % options)
+
     fp = savefig
     if savefig is None:
         if animated:
             f, fp = tempfile.mkstemp(suffix='.gif')
         else:
-            from nisnap.snap import format
-            f, fp = tempfile.mkstemp(suffix=format)
-        os.close(f)
+            from nisnap.snap import __format__
+            f, fp = tempfile.mkstemp(suffix=__format__)
+        __os__.close(f)
 
     dest = tempfile.gettempdir()
     # Downloading resources
@@ -366,11 +435,17 @@ def plot_segment(config, experiment_id, savefig=None, slices=None,
     except FileNotFoundError as exc:
         msg = '{0}. Retry with cache set to False.'.format(exc)
         raise FileNotFoundError(msg)
-
     bg = filepaths[0]
 
-    filepaths = filepaths[1] if len(filepaths) == 2 else filepaths[1:]
+    if resource_name == 'FREESURFER7_EXTRAS':
+        filepaths = [bg, filepaths[options.index(fn) + 1]]
+    elif resource_name.startswith('FREESURFER'):
+        if fn[3:].startswith('hippoAmygLabels'):
+            filepaths = [bg, filepaths[options.index(fn) + 1]]
+        else:
+            filepaths = filepaths[:2]
 
+    filepaths = filepaths[1] if len(filepaths) == 2 else filepaths[1:]
     from . import snap
     snap.plot_segment(filepaths, axes=axes, bg=bg, opacity=opacity,
                       animated=animated, savefig=fp, figsize=figsize,
